# tsp-ga

## Running the program
From the top-level directory, run a command like this:
```
$ stack run 100 100 5 100 100 -- +RTS -s -N5
```

You can print the usage text by running it with a different number of arguments:
```
$ stack run
Usage: stack run <numCities> <popSize> <k> <n> <m> -- +RTS -s -N<cores>

numCities - The number of cities to traverse
popSize   - The number of candidate paths that are evaluated during each generation
k         - The number of branches of generations to evaluate in parallel
n         - The number of generations to evaluate within a branch before merging with the other branches
m         - The number of times to split into parallel branches and then merge the results
cores     - The number of cores to run the program with
```

## Interpreting outputs
* If this is the first time building the code, there will be some compiler outputs
* Then there will be a table that gradually fills in with stats about every Nth generation.  The "Min Path Length"
  column should get better in each passing generation, since we preserve the best 20 paths from one generation to
  the next.  The table should look like this:
  ```
  Generation | Min Path Length    | Average Path Length   | Max Path Length
  -----------|--------------------|-----------------------|----------------------
  0          | 17119.840743411183 | 19707.3978924783      | 21440.364742743583
  100        | 8239.888624958845  | 8775.736764993506     | 9079.823415591309
  200        | 6049.031047285989  | 6443.558211610912     | 6735.235319888242
  300        | 5501.5258485583745 | 5596.075019748084     | 5637.752872506867
  400        | 4996.729566087036  | 5160.697416243478     | 5245.606629301489
  500        | 4747.978911806753  | 4786.325340944778     | 4822.2019657926
  ```
* Next there is a JSON string, like this, that contains info about the absolute best path that was found:
  ```
  {
    "cities": [[281,340],[164,147],[50,65],[182,326],[9,175],[44,125],[116,186],[177,193],[32,383],[292,29]],
    "pathIndices": [9,1,6,7,0,3,8,4,5,2],
    "score": 1313.1210556605738
  }
  ```
  See [below](#viewing-the-generated-path) for instructions on how to view this path in a browser.
* Finally, there are outputs from GHC that describes the runtime behavior, such as the total elapsed time
  and the amount of time spent on GC.

## Accessing the reference solution
* View the [code](https://github.com/CodingTrain/Coding-Challenges/tree/main/035.4_TSP_GA/P5) in GitHub.
* View and run the [code](https://editor.p5js.org/codingtrain/sketches/EGjTrkkf9) in the P5JS web editor.
  Press the play button in the top right to see a simulation of the genetic algorithm.  In the panel on
  the right, the path on the top is the best path seen so far and the path on the bottom is the best path
  of the current generation.  To see all of the files, click the arrow below the play button.
* If you have this repo checked out, run `git submodule init` and then `git submodule update` to clone
the repo that contains the reference solution into the `.submodules` directory.  Then, the
[`reference_solution`](reference_solution) symlink will bring you right to the correct subdirectory.

## Viewing the generated path
After running the program with `stack run`, a JSON string will be printed, containing a list of city
coordinates, a path (as a list of city indices), and the score of that path.  To visualize this path,
there is a simple P5JS sketch in the [`path_viewer/`](path_viewer) directory.  You can run this in the
[P5JS web editor](https://editor.p5js.org/timothy-johns/sketches/l_ZcOn_nN).  Simply copy the JSON
string that was output by the program into the text box when you run the sketch in the browser and
you'll see a drawing of the cities and the most optimal path that was found.
