module Main (main) where

import Lib
import System.Environment (getArgs)
import System.Exit (die)
import System.Random (mkStdGen)

main :: IO ()
main = do
  args <- getArgs
  case map read args of
    [numCities, popSize, k, n, m] -> do
      let r              = mkStdGen 12345
          (cities, r')   = createCities numCities r
          pathLength     = calcPathLength cities
          (pop, r'')     = initialPopulation cities popSize pathLength r'
          popStats       = runGA pop m k n pathLength r''

      putStrLn "Generation | Min Path Length    | Average Path Length   | Max Path Length"
      putStrLn "-----------|--------------------|-----------------------|----------------------"
      mapM_ (\(i, (PopulationStats _ minLen maxLen avgLen)) ->
               putStrLn $ show i      ++ "\t   | "
                       ++ show minLen ++ "\t| "
                       ++ show avgLen ++ "\t| "
                       ++ show maxLen
            ) $ zip [0,n..] popStats 

      putStrLn $ toJsonStr (TspResult cities (last popStats))

    _ -> do
      die $ "Usage: stack run <numCities> <popSize> <k> <n> <m> -- +RTS -s -N<cores>\n\n"
         ++ "numCities - The number of cities to traverse\n"
         ++ "popSize   - The number of candidate paths that are evaluated during each generation\n"
         ++ "k         - The number of branches of generations to evaluate in parallel\n"
         ++ "n         - The number of generations to evaluate within a branch before merging with the other branches\n"
         ++ "m         - The number of times to split into parallel branches and then merge the results\n"
         ++ "cores     - The number of cores to run the program with\n"
